/***************************************************************************
 *   Copyright (C) 2009 by Stanson                                         *
 *   stanson@btv.ru                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 3                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <math.h>

#include <usb.h>

#include <qlabel.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qpixmap.h>

#include "qoscope.h"
#include "QOscilloscope.h"

#include <QTimerEvent>
#include <QPaintEvent>

#define USB_OSC_VENDOR_ID	0x1d50
#define USB_OSC_PRODUCT_ID	0x6030
//#define USB_OSC_VENDOR_ID	0x04d8
//#define USB_OSC_PRODUCT_ID	0x0002

#define READ_VERSION            0
#define COLLECT_OSC_DATA        1
#define READ_OSC_DATA           2
#define SET_OFFSET              3
#define SET_TIMESCALE           4
#define SET_SYNC                5

QOscope::QOscope()
 : QWidget()
{

    setGeometry( 0, 0, 266, 266 );
    setMaximumSize( 266, 266 );
    setMinimumSize( 266, 266 );
    memset(data, 127, sizeof(data));
}


QOscope::~QOscope()
{
    if( dev )
    {
	usb_release_interface( dev, 0 );
	usb_close( dev );
    }
}

void QOscope::initHardware()
{
    struct usb_bus 	*usbbus;
    struct usb_device	*usbdev;
    int ret;
    char ps[128], sn[128], buffer[256];

    dev = NULL;

    usb_init();
    usb_find_busses();
    usb_find_devices();

    usbbus = usb_get_busses();
    for (; usbbus; usbbus = usbbus->next)
    {
	for (usbdev = usbbus->devices; usbdev; usbdev=usbdev->next)
	{
	    if (usbdev->descriptor.idVendor == USB_OSC_VENDOR_ID &&
	        usbdev->descriptor.idProduct == USB_OSC_PRODUCT_ID) 
	    {
		dev = usb_open(usbdev);
		if(!dev)
		{
		    fprintf(stderr, "Failed to open USB device %04x:%04x\n", USB_OSC_VENDOR_ID, USB_OSC_PRODUCT_ID);
		    ret = -1;
		    goto no_device;
		}
		ret = usb_claim_interface(dev, 0);
		if(ret < 0)
		{
		    ret = usb_detach_kernel_driver_np(dev, 0);
		    if(ret < 0)
		    {
			fprintf(stderr, "Failed to detach kernel driver from interface 0 on device %04x:%04x\n", USB_OSC_VENDOR_ID, USB_OSC_PRODUCT_ID);
			goto no_device;
		    }
		    ret = usb_claim_interface(dev, 0);
		    if(ret < 0)
		    {
			fprintf(stderr, "Failed to claim interface 0 on device %04x:%04x\n", USB_OSC_VENDOR_ID, USB_OSC_PRODUCT_ID);
			goto no_device;
		    }
		}
		usb_get_string_simple(dev, 2, ps, sizeof(ps));
		usb_get_string_simple(dev, 3, sn, sizeof(sn));
		snprintf(buffer, sizeof(buffer), "Hardware : %s %s", ps, sn);
		emit haveMessage( buffer );
	        goto device_found;
	    }
        }
    }

no_device:
    if( dev )
    {
	usb_close( dev );
	dev = NULL;
    }
    fprintf(stderr, "Device %04x:%04x not found!\n", USB_OSC_VENDOR_ID, USB_OSC_PRODUCT_ID);

device_found:

    setOffset(8);
    setTimescale(2);
    setSync(0);

    startTimer( 50 );
}

void QOscope::timerEvent( QTimerEvent * )
{
    int i, j, x = 0, ret;

    if( dev )
    {
	pkt[0] = COLLECT_OSC_DATA;

	usb_bulk_write(dev, USB_ENDPOINT_OUT | 1, (char *)&pkt, 1, 1000);
	usb_bulk_read(dev, USB_ENDPOINT_IN | 1, (char *)&pkt, 1, 1000);

	for(i = 0; i < 4; i++)
	{
    	    pkt[0] = READ_OSC_DATA;
    	    pkt[1] = i;

	    usb_bulk_write(dev, USB_ENDPOINT_OUT | 1, (char *)&pkt, 2, 1000);
	    usb_bulk_read(dev, USB_ENDPOINT_IN | 1, (char *)&pkt, 64, 1000);

    	    for(j = 0; j < 64; j++, x++)
    	    {
        	data[x] = pkt[j];
    	    }
	}
    }

    repaint();
}

void QOscope::paintEvent( QPaintEvent * )
{
    int i = 0;
    int x1 = 0, y1 = 0;
    int x2 = 0, y2 = 0;
    int v = 0;
    double vavg = 0, vmax = 0, vmin = 255, vamp = 0;
    QString str;

    QPainter p;
    p.begin( this );
    p.fillRect( 0, 0, width(), height(), Qt::black );

    QColor light( 160, 160, 160 );
    QColor dark( 92, 92, 92 );

    for( i = 0; i < 11; i++ )
    {
      int y = 5 + i * 255 / 10;
      p.setPen( (i == 5) ? light : dark );
      p.drawLine( 5, y, 260, y );
    }

    for( i = 0; i < 11; i++ )
    {
      int x = 5 + i * 255 / 10;
      p.setPen( (i == 5) ? light : dark );
      p.drawLine( x, 5, x, 260 );
    }

    if( sync != 0 )
    {
      int y = 255 - round((sync + 50) * 2.55) + 5;
      p.setPen( Qt::red );
      p.drawLine( 2, y, 8, y );
    }

    p.setPen( Qt::green );
//    p.setRenderHint( QPainter::Antialiasing, true );

    for(i = 0; i < 256; i++)
    {
      x2 = i + 5;
      y2 = 255 - data[i] + 5;
      if(i) p.drawLine( x1, y1, x2, y2 );
      x1 = x2;
      y1 = y2;
      v = data[i] - 128;
      if( v > vmax ) vmax = v;
      if( v < vmin ) vmin = v;
      vavg += v;
    }
    vavg = vavg / ( 256.0 * 25.6 );
    vmax = vmax / 25.6 ;
    vmin = vmin / 25.6 ;
    vamp = vmax - vmin ;

    str = QString("Vavg: %1").arg( vavg, 0, 'f', 2 );
    p.drawText( 8, 8, 83, 16, Qt::AlignLeft, str );
    str = QString("Vmin: %1").arg( vmin, 0, 'f', 2 );
    p.drawText( 91, 8, 83, 16, Qt::AlignLeft, str );
    str = QString("Vmax: %1").arg( vmax, 0, 'f', 2 );
    p.drawText( 174, 8, 83, 16, Qt::AlignLeft, str );

    p.drawText( 91, 243, 83, 16, Qt::AlignCenter, txt_scale );
    str = QString("Trig: %1").arg( sync / 10.0, 0, 'f', 2 );
    p.drawText( 174, 243, 83, 16, Qt::AlignCenter, str );

    p.end();
}

void QOscope::setTimescale( int v )
{
    timescale = 11 - v;
    switch(timescale)
    {
      case  0: txt_scale = "5 uS/div";   break;
      case  1: txt_scale = "10 uS/div";  break;
      case  2: txt_scale = "20 uS/div";  break;
      case  3: txt_scale = "50 uS/div";  break;
      case  4: txt_scale = "0.1 mS/div"; break;
      case  5: txt_scale = "0.2 mS/div"; break;
      case  6: txt_scale = "0.5 mS/div"; break;
      case  7: txt_scale = "1 mS/div";   break;
      case  8: txt_scale = "2 mS/div";   break;
      case  9: txt_scale = "5 mS/div";   break;
      case 10: txt_scale = "10 mS/div";  break;
      case 11: txt_scale = "20 mS/div";  break;
    }
    pkt[0] = SET_TIMESCALE;
    pkt[1] = timescale;
    if( dev ) usb_bulk_write(dev, USB_ENDPOINT_OUT | 1, (char *)&pkt, 2, 1000);
    repaint();
}

void QOscope::setOffset( int v )
{
    offset = 15 - v;
    pkt[0] = SET_OFFSET;
    pkt[1] = offset;
    if( dev ) usb_bulk_write(dev, USB_ENDPOINT_OUT | 1, (char *)&pkt, 2, 1000);
}

void QOscope::setSync( int v )
{
    sync = v;
    pkt[0] = SET_SYNC;
    pkt[1] = round((sync + 50) * 2.56);
    if( dev ) usb_bulk_write(dev, USB_ENDPOINT_OUT | 1, (char *)&pkt, 2, 1000);
    repaint();
}

#include "qoscope.moc"
